#include "audio_device_manager.h"

#include "audio/mixer/mixer.h"
#include "audio/mixer/mixer_track.h"
#include "audio/plugin_host/midi_sequencer.h"
#include "audio/plugin_host/plugin_host_manager.h"
#include "thread_message/thread_message_hubs.h"

int rt_audio_callback(void *output_buffer, void *input_buffer,
                                unsigned int frames_nums,
                                double stream_time,
                                RtAudioStreamStatus status,
                                void* user_data)
{
    auto* API = static_cast<audio_device_manager*>(user_data);
    return API->stream_callback((float**)output_buffer, static_cast<float**>(input_buffer), frames_nums, stream_time, status);
}

void audio_device_manager::init(singleton_initliazer& initliazer) {
    singleton_t<audio_device_manager>::init(initliazer);
    options_.flags = RTAUDIO_NONINTERLEAVED;
}

void audio_device_manager::release(singleton_release_guard& release_guard) {
    singleton_t<audio_device_manager>::release(release_guard);
    release_guard.require_release<mixer>();
}

void audio_device_manager::start() {
    RtAudio::StreamParameters* input_params = nullptr;
    if (input_params_.deviceId != 0) {
        input_params = &input_params_;
    }
    // audio_.openStream(&output_params_, input_params, RTAUDIO_FLOAT32, get_sample_rate(), &options_.numberOfBuffers, &rt_audio_callback, this, &options_);
}

uint32_t audio_device_manager::get_sample_rate() const {
    return sample_rate_;
}

void audio_device_manager::set_sample_rate(uint32_t sample_rate) {
    sample_rate_ = sample_rate;
}

uint32_t audio_device_manager::get_buffer_size() const {
    return options_.numberOfBuffers;
}

void audio_device_manager::set_buffer_size(int buffer_size) {
    options_.numberOfBuffers = buffer_size;
}

uint32_t audio_device_manager::get_input_channel_count() const {
    return 0;
}

uint32_t audio_device_manager::get_output_channel_count() const {
    return 2; // 现在是固定值, 以后可能会改
}

int audio_device_manager::stream_callback(float** output, float** input, unsigned long frame_count, double stream_time,
    RtAudioStreamStatus status) {
    if (render_buffer_.Num() < frame_count * output_params_.nChannels)
        return 0;
    for (int i = 0; i < output_params_.nChannels; ++i) {
        render_buffer_.Pop(input[i], frame_count);
    }
    return 0;
}

void audio_device_manager::start_render_thread() {
    render_thread_ = std::thread(&audio_device_manager::render_thread, this);
    render_thread_.detach();
}

void audio_device_manager::stop_render_thread() {
    if (render_thread_.joinable())
        render_thread_.join();
}

void audio_device_manager::render_thread() {
    dummy_track* master = g_mixer.get_master();
    spdlog::info("port_audio render thread started");

    render_buffer_.SetCapacity(get_buffer_size() * 4);
    const uint32_t frames = get_buffer_size();
    const uint32_t rate = get_sample_rate();

    // while (audio_.isStreamRunning()) {
    //     const uint32_t milliseconds = 1.f / (rate / (frames / 4)) * 1000;
    //
    //     g_audio_thread_hub.process_messages();
    //     if (render_buffer_.Num() >= render_buffer_size_) {
    //         std::this_thread::sleep_for(std::chrono::milliseconds(milliseconds));
    //         continue;
    //     }
    //     g_midi_sequencer.process(rate, frames);
    //     g_mixer.reset();
    //     g_plugin_host_manager.process(frames);
    //     g_mixer.process(frames);
    //
    //     const auto& master_headers = master->buffer.get_headers_vector();
    //     for (const auto element: master_headers) {
    //         render_buffer_.Push(element, frames);
    //     }
    // }
    spdlog::info("port_audio render thread stopped");
}
