#pragma once
#include "misc/singleton/singleton.h"

#include "audio/misc/circular_audio_buffer.h"
#include "RtAudio.h"

class audio_device_manager : public singleton_t<audio_device_manager> {
public:
    void init(singleton_initliazer& initliazer) override;
    void release(singleton_release_guard& release_guard) override;

    void start();

    CORE_API const char* get_name() override { return "audio_device_manager"; }

    CORE_API uint32_t get_sample_rate() const;
    CORE_API void set_sample_rate(uint32_t sample_rate);
    CORE_API uint32_t get_buffer_size() const;
    CORE_API void set_buffer_size(int buffer_size);
    CORE_API uint32_t get_input_channel_count() const;
    CORE_API uint32_t get_output_channel_count() const;

    int stream_callback(float** output, float** input, unsigned long frame_count, double stream_time, RtAudioStreamStatus status);
protected:
#pragma region render_thread
    void start_render_thread();
    void stop_render_thread();
    void render_thread();
    std::thread render_thread_;
    circular_audio_buffer<float> render_buffer_;
    uint32_t render_buffer_size_ = 512;
    std::atomic_bool render_thread_running_ = false;
#pragma endregion
private:
    RtAudio::StreamParameters input_params_ = {};
    RtAudio::StreamParameters output_params_ = {};
    RtAudio::StreamOptions options_ = {};
    uint32_t sample_rate_ = 44100;
    // RtAudio audio_;
};

DEFINE_SINGLETON_INSTANCE(audio_device_manager)
