#pragma once
#include "audio/misc/audio_buffer.h"
#include "audio/misc/circular_audio_buffer.h"
#include "misc/delegates.h"

class channel_interface;
class mixer_track;
class plugin_host;

enum class mixer_track_type {
    unknown,
    dummy,
    instrument,
};

struct mixer_track_link {
    mixer_track* track;
    float send_level;
};

class mixer_track {
public:
    explicit mixer_track(mixer_track_type in_type) : type_(in_type) {}
    virtual ~mixer_track();

    void init();
    void add_effect(plugin_host* in_effect);
    void remove_effect(plugin_host* in_effect);
    void add_child(mixer_track* in_child, float in_send_level);
    void remove_child(mixer_track* in_child);

    void process(uint32_t in_frames);

    virtual void rename(const std::string& in_name) = 0;
    [[nodiscard]] virtual std::string get_name() const = 0;

    [[nodiscard]] float** get_headers() { return buffer.get_headers(); }

    [[nodiscard]] mixer_track_type get_type() const { return type_; }

    [[nodiscard]] channel_interface* get_channel_interface() const { return channel_interface_; }

    audio_buffer buffer;
    std::vector<circular_audio_buffer<float>> ui_buffers;
    std::atomic<float> volume = 1.0f;
    std::vector<plugin_host*> effects{};
    std::vector<mixer_track_link> children{};
    multicast_delegate<mixer_track*> on_processed;
private:
    void add_effect_internal(plugin_host* in_effect);
    const mixer_track_type type_;
    channel_interface* channel_interface_ = nullptr;
};

class instrument_track : public mixer_track {
public:
    instrument_track(plugin_host* in_instrument) : mixer_track(mixer_track_type::instrument), instrument_(in_instrument) {}

    void rename(const std::string& in_name) override;
    [[nodiscard]] std::string get_name() const override;

    [[nodiscard]] plugin_host* get_instrument() const { return instrument_; }
private:
    plugin_host* instrument_;
};

class dummy_track : public mixer_track {
public:
    dummy_track() : mixer_track(mixer_track_type::dummy) {}

    void rename(const std::string& in_name) override { name_ = in_name; }
    [[nodiscard]] std::string get_name() const override { return name_.c_str(); }
private:
    std::string name_;
};
